using System;
using System.Collections.Generic;
using System.Text;
using System.Web.UI;
using System.Collections;
using System.ComponentModel;

namespace Jeffz.AtlasControlToolkit
{
	[TypeConverter(typeof(ExpandableObjectConverter))]
	public sealed class Image : IStateManager
	{
		private bool _IsTrackingViewState = false;
		private StateBag _ViewState;

		public Image()
			: this("", null, null)
		{ }

		public Image(string url, int? width, int? height)
		{
			this.Url = url;
			this.Width = width;
			this.Height = height;
		}

		public string Url
		{
			get
			{
				string url = (string)(this.ViewState["Url"]);
				return (url == null) ? "" : url;
			}
			set
			{
				this.ViewState["Url"] = value;
			}
		}

		public int? Width
		{
			get
			{
				return (int?)(this.ViewState["Width"]);
			}
			set
			{
				this.ViewState["Width"] = value;
			}
		}

		public int? Height
		{
			get
			{
				return (int?)(this.ViewState["Height"]);
			}
			set
			{
				this.ViewState["Height"] = value;
			}
		}

		#region IStateManager Members

		bool IStateManager.IsTrackingViewState
		{
			get
			{
				return this._IsTrackingViewState;
			}
		}

		void IStateManager.LoadViewState(object state)
		{
			if (state != null)
			{
				((IStateManager)this.ViewState).LoadViewState(state);
			}
		}

		object IStateManager.SaveViewState()
		{
			if (this._ViewState != null)
			{
				this.SetDirty();
				return ((IStateManager)this._ViewState).SaveViewState();
			}

			return null;
		}

		void IStateManager.TrackViewState()
		{
			this._IsTrackingViewState = true;

			if (this._ViewState != null)
			{
				((IStateManager)this._ViewState).TrackViewState();
			}
		}

		#endregion

		private StateBag ViewState
		{
			get
			{
				if (this._ViewState == null)
				{
					this._ViewState = new StateBag(false);

					if (this._IsTrackingViewState)
					{
						((IStateManager)this._ViewState).TrackViewState();
					}
				}

				return this._ViewState;
			}
		}

		internal void SetDirty()
		{
			if (this._ViewState != null)
			{
				this._ViewState.SetDirty(true);
			}
		}

	}
}
