using System;
using System.Collections.Generic;
using System.Text;
using System.Collections;
using System.Web.UI;

namespace Jeffz.AtlasControlToolkit
{
	public class ImageCollection : IList, IStateManager
	{
		private List<Image> imageList = new List<Image>();
		private bool _IsTrackingViewState = false;

		internal ImageCollection() { }

		public Image this[int index]
		{
			get
			{
				return this.imageList[index];
			}
			set
			{
				if (value == null)
				{
					throw new ArgumentException("Please provide a valid Image", "item");
				}

				this.imageList[index] = value;
				if (this._IsTrackingViewState)
				{
					((IStateManager)value).TrackViewState();
				}
			}
		}

		public int Add(Image item)
		{
			if (item == null)
			{
				throw new ArgumentException("Please provide a valid Image", "item");
			}

			this.imageList.Add(item);
			if (this._IsTrackingViewState)
			{
				((IStateManager)item).TrackViewState();
				item.SetDirty();
			}

			return this.imageList.Count - 1;
		}

		public void Clear()
		{
			this.imageList.Clear();
		}

		public void Insert(int index, Image item)
		{
			if (item == null)
			{
				throw new ArgumentException("Please provide a valid Image", "item");
			}

			this.imageList.Insert(index, item);
			if (this._IsTrackingViewState)
			{
				((IStateManager)item).TrackViewState();
			}
		}

		public void Remove(Image item)
		{
			if (item == null)
			{
				throw new ArgumentException("Please provide a valid Image", "item");
			}

			this.imageList.Remove(item);
		}

		public void RemoveAt(int index)
		{
			this.imageList.RemoveAt(index);
		}

		public bool Contains(Image item)
		{
			if (item == null)
			{
				throw new ArgumentException("Please provide a valid Image", "item");
			}

			return this.imageList.Contains(item);
		}

		public int IndexOf(Image item)
		{
			return this.imageList.IndexOf(item);
		}

		public IEnumerator GetEnumerator()
		{
			return this.imageList.GetEnumerator();
		}

		#region IStateManager Members

		bool IStateManager.IsTrackingViewState
		{
			get
			{
				return this._IsTrackingViewState;
			}
		}

		void IStateManager.LoadViewState(object state)
		{
			ArrayList stateList = state as ArrayList;
			this.imageList.Clear();

			if (stateList != null)
			{
				foreach (object s in stateList)
				{
					Image image = new Image();
					this.imageList.Add(image);
					((IStateManager)image).LoadViewState(s);
				}
			}
		}

		object IStateManager.SaveViewState()
		{
			ArrayList stateList = new ArrayList(this.imageList.Count);
			foreach (Image image in this.imageList)
			{
				stateList.Add(((IStateManager)image).SaveViewState());
			}

			return stateList;
		}

		void IStateManager.TrackViewState()
		{
			this._IsTrackingViewState = true;
			foreach (Image image in this.imageList)
			{
				((IStateManager)image).TrackViewState();
			}
		}

		#endregion

		#region IList Members

		int IList.Add(object value)
		{
			return this.Add(value as Image);
		}

		bool IList.Contains(object value)
		{
			return this.Contains(value as Image);
		}

		int IList.IndexOf(object value)
		{
			return this.IndexOf(value as Image);
		}

		void IList.Insert(int index, object value)
		{
			this.Insert(index, value as Image);
		}

		bool IList.IsFixedSize
		{
			get
			{
				return false;
			}
		}

		bool IList.IsReadOnly
		{
			get
			{
				return false;
			}
		}

		void IList.Remove(object value)
		{
			this.Remove(value as Image);
		}

		void IList.RemoveAt(int index)
		{
			this.RemoveAt(index);
		}

		object IList.this[int index]
		{
			get
			{
				return this[index];
			}
			set
			{
				this[index] = value as Image;
			}
		}

		#endregion

		#region ICollection Members

		void ICollection.CopyTo(Array array, int index)
		{
			throw new Exception("The method or operation is not implemented.");
		}

		public int Count
		{
			get
			{
				return this.imageList.Count;
			}
		}

		bool ICollection.IsSynchronized
		{
			get { throw new Exception("The method or operation is not implemented."); }
		}

		object ICollection.SyncRoot
		{
			get { throw new Exception("The method or operation is not implemented."); }
		}

		#endregion

	}
}
