Type.registerNamespace('Jeffz.AtlasControlToolkit');

Jeffz.AtlasControlToolkit.ImageGalleryBehavior = function()
{
	Jeffz.AtlasControlToolkit.ImageGalleryBehavior.initializeBase(this);
	
	var isIE = (Sys.Runtime.get_hostType() == Sys.HostType.InternetExplorer);
	
	var m_preloadImageQueue;
	var m_preloadImage;
	var m_galleryControl = null;
	var m_waitingControl = null;
	var m_controlEle = null;
	var m_currIndex = null;
	var m_currImage = null;
	var m_nextImage = null;
	var m_loadingImage = null;
	var m_timeoutFeed = null;
	var m_isSwitching = false;
	var m_currRealWidth = null;
	var m_currRealHeight = null;
	
	this.initialize = function()
	{
		Jeffz.AtlasControlToolkit.ImageGalleryBehavior.callBaseMethod(this, 'initialize');
		
		this.control.get_Interval = this.get_Interval;
		this.control.set_Interval = this.set_Interval;
		this.control.get_SwitchTime = this.get_SwitchTime;
		this.control.set_SwitchTime = this.set_SwitchTime;
		this.control.next = next;
		this.control.previous = previous;
		this.control.select = select;
		this.control.adjustImage = function()
		{
			adjustImagePosition(m_currImage, m_currRealWidth, m_currRealHeight);
		}
		
		preloadImagesForCache();
		
		m_waitingControl = document.getElementById(_WaitingControlID);
		m_galleryControl = this.control.element;
		initImageGallery();
	}
	
	this.dispose = function()
	{
		m_preloadImageQueue = null;
		m_preloadImage = null;
		m_galleryControl = null;
		m_waitingControl = null;
		m_currIndex = null;
		m_currImage = null;
		m_currImage = null;
		m_loadingImage = null;
		m_isSwitching = null;
		m_timeoutFeed = null;
		m_currRealWidth = null;
		m_currRealHeight = null;
		
		Jeffz.AtlasControlToolkit.ImageGalleryBehavior.callBaseMethod(this, 'dispose');
	}
	
	this.getDescriptor = function()
	{
		var td = Jeffz.AtlasControlToolkit.ImageGalleryBehavior.callBaseMethod(this, 'getDescriptor');
	
		td.addProperty('Images', Array);
		td.addProperty('Interval', Number);
		td.addProperty('WaitingControlID', String);
		td.addProperty("SwitchTime", Number);
		return td;
	}
	
	var _Images = null;
	this.get_Images = function()
	{
		return _Images;
	}
	this.set_Images = function(value)
	{
		_Images = value;
	}
	
	var _Interval = 0;
	this.get_Interval = function()
	{
		return _Interval;
	}
	this.set_Interval = function(value)
	{
		_Interval = value;
	}
	
	var _WaitingControlID
	this.get_WaitingControlID = function()
	{
		return _WaitingControlID;
	}
	this.set_WaitingControlID = function(value)
	{
		_WaitingControlID = value;
	}
	
	var _SwitchTime = 0;
	this.get_SwitchTime = function()
	{
		return _SwitchTime;
	}
	this.set_SwitchTime = function(value)
	{
		_SwitchTime = value;
	}
	
	
	function preloadImagesForCache()
	{
		m_preloadImageQueue = new Array();
		for (var i = 0; i < _Images.length; i++)
		{
			m_preloadImageQueue.push(_Images[i].Url);
		}
		
		m_preloadImage = new Image();
		m_preloadImage.onload = onload;
		m_preloadImage.onerror = onload;
		m_preloadImage.src = m_preloadImageQueue.shift();
		
		function onload()
		{
			if (m_preloadImageQueue.length > 0)
			{
				m_preloadImage.src = m_preloadImageQueue.shift();
			}
		}
	}
	
	function initImageGallery()
	{
		m_currImage = document.createElement("img");
		m_currImage.style.position = "absolute";
		m_currImage.style.display = "none";
		m_galleryControl.appendChild(m_currImage);
		
		m_nextImage = document.createElement("img");
		m_nextImage.style.position = "absolute";
		m_nextImage.style.display = "none";
		m_galleryControl.appendChild(m_nextImage);
		
		if (m_waitingControl)
		{
			m_waitingControl.style.visibility = "hidden";
		}
		
		m_loadingImage = new Image();
		m_loadingImage.onload = loadImageComplete;
	}
	
	
	function next()
	{
		if (m_isSwitching)
		{
			return;
		}
		else
		{
			m_isSwitching = true;
		}
		
		if (m_currIndex == null)
		{
			m_currIndex = 0;
		}
		else
		{
			m_currIndex = (m_currIndex + 1) % _Images.length;
		}
		loadCurrentImage();
	}
	
	function previous()
	{
		if (m_isSwitching)
		{
			return;
		}
		else
		{
			m_isSwitching = true;
		}
		
		if (m_currIndex == null)
		{
			m_currIndex = 0;
		}
		else
		{
			m_currIndex = (m_currIndex - 1 + _Images.length) % _Images.length;
		}
		loadCurrentImage();
	}
	
	function select(index)
	{
		if (m_isSwitching)
		{
			return;
		}
		else
		{
			m_isSwitching = true;
		}
		
		index = Math.abs(index) % _Images.length;
		if (m_currIndex != index)
		{
			m_currIndex = index;
			loadCurrentImage();
		}
		else
		{
			m_isSwitching = false;
		}
	}
	
	function loadCurrentImage()
	{
		if (m_timeoutFeed)
		{
			clearTimeout(m_timeoutFeed);
			m_timeoutFeed = null;
		}
		
		if (m_waitingControl)
		{
			m_waitingControl.style.visibility = "visible";
		}
		
		m_loadingImage.src = _Images[m_currIndex].Url;
	}
	
	function loadImageComplete()
	{
		if (m_waitingControl)
		{
			m_waitingControl.style.visibility = "hidden";
		}
		
		adjustAndShowNextImage();
		
		function adjustAndShowNextImage()
		{
			// get the image's size
			var width = _Images[m_currIndex].Width;
			var height = _Images[m_currIndex].Height;
			if (width == null && height == null) // use the image size;
			{
				width = m_loadingImage.width; 
				height = m_loadingImage.height;
			}
			else if (width != null && height == null) // get height from width;
			{
				height = width * 1.0 * m_loadingImage.height / m_loadingImage.width;
			}
			else if (width == null && height != null) // get width from height;
			{
				width = height * 1.0 * m_loadingImage.width / m_loadingImage.height;
			}
			
			// adjust the position
			m_currRealWidth = width;
			m_currRealHeight = height;
			
			adjustImagePosition(m_nextImage, width, height);
			m_nextImage.src = m_loadingImage.src;
			m_nextImage.style.display = "block";
			
			var tempImage = m_nextImage;
			m_nextImage = m_currImage;
			m_currImage = tempImage;
			
			new TransHelper(m_nextImage, _SwitchTime, false);
			new TransHelper(m_currImage, _SwitchTime, true);
		}
	}
	
	function TransHelper(img, time, isIn)
	{
		if (time <= 0)
		{
			if (isIn)
			{
				img.style.filter = "alpha(opacity=100)";
				img.style.opacity = "1";
			}
			else
			{
				img.style.filter = "alpha(opacity=0)";
				img.style.opacity = "0";
			}
			
			displayFinished();
			return;
		}
	
		var interval = 120;
		var totalTimes = time * 3;
		var currTimes;
		
		if (isIn) // opacity: 0 -> 100
		{
			img.style.filter = "alpha(opacity=0)";
			img.style.opacity = "0";
			currTimes = 0;
			setTimeout(increase, interval);
		}
		else // opacity: 100 -> 0
		{
			img.style.filter = "alpha(opacity=100)";
			img.style.opacity = "1";
			currTimes = totalTimes;
			setTimeout(decrease, interval);
		}
		
		function increase()
		{
			currTimes ++;
			img.style.filter = "alpha(opacity=" + currTimes * 100 / totalTimes + ")";
			img.style.opacity = (currTimes * 1.0 / totalTimes) + "";
			
			if (currTimes < totalTimes)
			{
				setTimeout(increase, interval);
			}
			else
			{
				displayFinished();
			}
		}
		
		function decrease()
		{
			currTimes --;
			img.style.filter = "alpha(opacity=" + currTimes * 100 / totalTimes + ")";
			img.style.opacity = (currTimes * 1.0 / totalTimes) + "";
			
			if (currTimes > 0)
			{
				setTimeout(decrease, interval);
			}
		}
	}
	
	function displayFinished()
	{
		m_isSwitching = false;
		if (_Interval > 0)
		{
			m_timeoutFeed = setTimeout(next, _Interval * 1000);
		}
	}
	
	function adjustImagePosition(imgEle, width, height)
	{
		// adjust image by gallery panel
		var galleryWidth = m_galleryControl.offsetWidth;
		var galleryHeight = m_galleryControl.offsetHeight;
		
		if (width == null)
		{
			width = imgEle.width;
		}
		if (height == null)
		{
			height = imgEle.height;
		}
		
		var t = 1;
		if (width > galleryWidth && height <= galleryHeight)
		{
			t = galleryWidth * 1.0 / width;
		}
		else if (width <= galleryWidth && height > galleryHeight)
		{
			t = galleryHeight * 1.0 / height;
		}
		else if (width > galleryWidth && height >= galleryHeight)
		{
			if (width * galleryHeight > height * galleryWidth) // width / galleryWidth > height / galleryHeight
			{
				t = galleryWidth * 1.0 / width;
			}
			else if (width * galleryHeight < height * galleryWidth) // width / galleryWidth < height / galleryHeight
			{
				t = galleryHeight * 1.0 / height;
			}
		}
		
		width = Math.floor(width * t);
		height = Math.floor(height * t);
		
		imgEle.width = width;
		imgEle.height = height;
		
		imgEle.style.top = (galleryHeight - height) / 2 + "px";
		imgEle.style.left = (galleryWidth - width) / 2 + "px";
	}

	// These are helper functions for communicating state back to the extender on the
	// server side.  They take or return a custom string that is available in your initialize method
	// and later.
	this.getClientState = function()
	{
		var value = Jeffz.AtlasControlToolkit.ImageGalleryBehavior.callBaseMethod(this, 'get_ClientState');		
		if (value == '') value = null;
		return value;
	}
	 
	this.setClientState = function(value)
	{
		return Jeffz.AtlasControlToolkit.ImageGalleryBehavior.callBaseMethod(this, 'set_ClientState',[value]);		
	}
}
Jeffz.AtlasControlToolkit.ImageGalleryBehavior.registerSealedClass('Jeffz.AtlasControlToolkit.ImageGalleryBehavior', AtlasControlToolkit.BehaviorBase);
Sys.TypeDescriptor.addType('Jeffz.AtlasControlToolkit'.toLowerCase() /* Safari Compat */, 'ImageGalleryBehavior', Jeffz.AtlasControlToolkit.ImageGalleryBehavior);
